# coding: utf-8
import tr
import os
from PIL import Image, ImageDraw
import numpy as np


class TRInstance():
    def __init__(self):
        pass

    def resize_img(self, img: Image, max_length=1200):
        img = img.convert("RGB")
        width, height = img.size
        max_edge = max(width, height)
        if max_edge <= max_length:
            return (img, 1)
        ratio = max_length * 1.0 / max_edge
        new_width = int(width * ratio)
        if new_width > max_length:
            new_width = max_length
        new_height = int(height * ratio)
        if new_height > max_length:
            new_height = max_length
        new_img = img.resize((new_width, new_height))
        return (new_img, ratio)

    def resize_p_img(self, img: Image, max_length=1200):
        new_img, ratio = self.resize_img(img, max_length)
        cv_img = np.asarray(new_img)
        back_img = np.zeros((max_length, max_length, 3))
        cv_shape = cv_img.shape
        back_img[0:cv_shape[0], 0:cv_shape[1], 0:cv_shape[2]] = cv_img
        res = Image.fromarray(back_img.astype('uint8'))
        return res, new_img, ratio

    def bbox_ocr_infos(self, img_pil, max_length=1600, ocr=False):
        width, height = img_pil.size
        res_img, new_img, ratio = self.resize_p_img(img_pil, max_length)
        gray_pil = res_img.convert("L")

        bbox_list = []
        score_list = []
        text_list = []
        res = {}
        if ocr:
            results = tr.run(gray_pil, flag=tr.FLAG_ROTATED_RECT)
        else:
            results = tr.detect(gray_pil, flag=tr.FLAG_ROTATED_RECT)
            results = np.asarray(results)
            if len(results) > 0:
                shape = results.shape
                results = results.reshape((shape[0], 1, shape[1]))
        for i, rect in enumerate(results):
            cx, cy, w, h, a = tuple(rect[0])
            if ratio < 1:
                cx = int(cx / ratio + 0.5)
                cy = int(cy / ratio + 0.5)
                w = int(w / ratio + 0.5)
                h = int(h / ratio + 0.5)
            # print(i, "\t", rect[1], rect[2])
            box = cv2.boxPoints(((cx, cy), (w, h), a))
            box = np.int0(np.round(box))

            bounding_box = cv2.boundingRect(np.array(box))
            bounding_box = list(bounding_box)
            if bounding_box[0] < 0:
                bounding_box[0] = 0
            if bounding_box[1] < 0:
                bounding_box[1] = 0
            if bounding_box[0] + bounding_box[2] > width:
                bounding_box[2] = width - bounding_box[0]
            if bounding_box[1] + bounding_box[3] > height:
                bounding_box[3] = height - bounding_box[1]

            bbox_list.append([bounding_box[0], bounding_box[1], bounding_box[0] + bounding_box[2],
                              bounding_box[1] + bounding_box[3]])
            if ocr:
                score_list.append(rect[2])
                text_list.append(rect[1])

        res['bboxes'] = bbox_list
        if ocr:
            res['confidence'] = score_list
            res['texts'] = text_list

        return res


if __name__ == "__main__":
    import random
    import cv2

    img_path = "/home/xiaohong/Pictures/A4/tmp/测评结果/img-test"
    save_path = "/home/xiaohong/Pictures/A4/tmp/测评结果/tr-res"
    intance = TRInstance()
    cnt = 0
    for file in os.listdir(img_path):
        print(file)
        # file = "选区_3429.jpg"
        img_pil = Image.open(os.path.join(img_path, file))
        cv_img = cv2.imread(os.path.join(img_path, file))
        img_draw = ImageDraw.Draw(img_pil)
        colors = ['red', 'green', 'blue', "purple"]
        res = intance.bbox_ocr_infos(img_pil, max_length=1024)
        for bbox in res['bboxes']:
            color = colors[random.randint(0, len(colors) - 1)]
            img_draw.rectangle((bbox[0], bbox[1], bbox[2], bbox[3]), fill=color, outline=(255, 0, 255), width=1)
            cv2.rectangle(cv_img, (bbox[0], bbox[1]), (bbox[2], bbox[3]), color=(255, 0, 0), thickness=1)
            cv2.imwrite(os.path.join(save_path, file), cv_img)
        img_pil.convert("RGB").save(os.path.join(save_path, "pil_" + file))
        cnt += 1
        # if cnt > 10:
        #     break
    print("---over---")
