import argparse
import logging
import os

import numpy as np
import torch
import torch.nn.functional as F
from PIL import Image
from torchvision import transforms
import cv2
import math
import onnxruntime


def resize_height(img : Image, dh=128):
    width, height = img.size
    ratio1 = height / dh
    new_height = dh
    new_width = int(width / ratio1 + 0.5)
    img = img.resize((new_width, new_height))
    return new_height, new_width, img

class UnetMasker():
    def __init__(self, model_path, img_height = 64):
        self.toTensor = transforms.ToTensor()
        self.onnx_sess = onnxruntime.InferenceSession(model_path)
        self.onnx_input_name = self.onnx_sess.get_inputs()[0].name
        self.onnx_output_name = self.onnx_sess.get_outputs()[0].name
        self.device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
        self.imgH = img_height

    def preprocess(self, pil_img: Image):
        img_ndarray = np.asarray(pil_img)
        img_ndarray = img_ndarray.transpose((2, 0, 1))
        img_ndarray = img_ndarray / 255
        return img_ndarray

    def get_mask(self, img: Image):
        img = img.convert("RGB")
        w, h, img = resize_height(img, self.imgH)
        img = torch.from_numpy(self.preprocess(img))
        img = img.unsqueeze(0).numpy()
        image_tensors = img.astype(np.float32)
        output = self.onnx_sess.run([self.onnx_output_name], {self.onnx_input_name: image_tensors})[0]
        probs = F.softmax(torch.from_numpy(output), dim=1)[0]
        mask = probs.numpy()[1]
        mask = mask*255
        mask = np.where(mask > 127, 255, 0)
        return mask


if __name__ == '__main__':

    # model_path = "./checkpoints/checkpoint_epoch41.pth"
    model_path = "/home/xiaohong/PythonProject2022/Pytorch-UNet-master/checkpoints/unet-p64-mod.onnx"
    unet_masker = UnetMasker(model_path)
    img = Image.open("/home/xiaohong/Pictures/ocr评估数据集/pic_82.png").convert("RGB")
    mask = unet_masker.get_mask(img)
    logging.info('Model loaded!')



